@@ -98,24 +98,6 @@ sub callid {
 	return $self->{callid};
 }
 
-############################################################################
-# gets contact, either from contact info on context (only for outgoing)
-# or from 'from'/'to'
-# Args: $self
-# Returns: $contact
-############################################################################
-sub contact {
-	my Net::SIP::Endpoint::Context $self = shift;
-	if ( $self->{incoming} ) {
-		my ($data) = sip_hdrval2parts( to => $self->{to} );
-		return $data;
-	} else {
-		return $self->{contact} if $self->{contact};
-		my ($data) = sip_hdrval2parts( from => $self->{from} );
-		return $data;
-	}
-}
-
 ############################################################################
 # get peer
 # Args: $self
@@ -143,7 +125,6 @@ sub new_request {
 	my Net::SIP::Endpoint::Context $self = shift;
 	my ($method,$body,%args) = @_;
 
-
 	my $request;
 	if ( ref($method)) {
 		# already a request object
@@ -170,7 +151,7 @@ sub new_request {
 		}
 
 		# contact is mandatory for INVITE
-		$contact = $from if $method eq 'INVITE' and ! defined $contact;
+		# will be added within Leg
 
 		$request = Net::SIP::Request->new(
 			$method,     # Method
@@ -178,7 +159,7 @@ sub new_request {
 			{
 				from => $from,
 				to => $to,
-				contact => $contact,
+				$contact ? ( contact => $contact ):(),
 				route => $self->{route},
 				cseq => "$cseq $method",
 				'call-id' => $self->{callid},
@@ -352,7 +333,14 @@ sub handle_response {
 		if ( $method eq 'INVITE' ) {
 			# is response to INVITE, create ACK
 			# and propagate to upper layer
-			my $ack = $tr->{request}->create_ack( $response );
+			my $req = $tr->{request};
+			if ( my $contact = $response->get_header( 'contact' )) {
+				# 12.1.2 - set URI for dialog to contact given in response which
+				# establishes the dialog
+				$contact = $1 if $contact =~m{<(\w+:[^>\s]+)>};
+				$req->set_uri( $contact );
+			}
+			my $ack = $req->create_ack( $response );
 			invoke_callback($cb,@arg,0,$code,$response,$leg,$from,$ack);
 			$endpoint->new_request( $ack,$self,undef,undef,leg => $leg, dst_addr => $from );
 
@@ -394,7 +382,8 @@ sub handle_response {
 		# 21.3.3 302 moved temporarily
 		# redo request and insert request again
 		my $contact = $self->{to} = $response->get_header( 'contact' );
-		( my $r = $tr->{request} )->set_header( to => $contact );
+		$contact = $1 if $contact =~m{<(\w+:[^>\s]+)>};
+		( my $r = $tr->{request} )->set_uri( $contact );
 		$r->set_cseq( ++$self->{cseq} );
 		$endpoint->new_request( $r,$self );
 
