@@ -5,7 +5,6 @@
 #    can save received data
 # - media_send_recv: receive and optionally save data. Sends back data
 #    from file with optional repeat count
-# only PCMU 8000 data will be handled at the moment
 ###########################################################################
 
 use strict;
@@ -17,6 +16,10 @@ use Net::SIP::Util qw(invoke_callback);
 use Socket;
 use Net::SIP::Debug;
 
+
+# on MSWin32 non-blocking sockets are not supported from IO::Socket
+use constant CAN_NONBLOCKING => $^O ne 'MSWin32';
+
 ###########################################################################
 # creates function which will initialize Media for echo back
 # Args: ($writeto,$delay)
@@ -47,7 +50,7 @@ sub media_recv_echo {
 			my @delay_buffer;
 			my $echo_back = sub {
 				my ($s_sock,$remote,$delay_buffer,$delay,$writeto,$targs,$didit,$sock) = @_;
-				while (1) {
+				for my $dummy (1) {
 					my $buf = _receive_rtp( $sock,$writeto,$targs,$didit );
 					defined($buf) or last;
 					#DEBUG( "$didit=$$didit" );
@@ -58,18 +61,19 @@ sub media_recv_echo {
 					while ( @$delay_buffer > $delay ) {
 						send( $s_sock,shift(@$delay_buffer),0,$remote );
 					}
+					CAN_NONBLOCKING && redo; # try recv again
 				}
 			};
 
 			$call->{loop}->addFD( $sock,
 				[ $echo_back,$s_sock,$addr,\@delay_buffer,$delay || 0,$writeto,{},\$didit ] );
-			my $was_blocking = $s_sock->blocking(0);
+			my $reset_to_blocking = CAN_NONBLOCKING && $s_sock->blocking(0);
 			push @{ $call->{ rtp_cleanup }}, [ sub {
-				my ($call,$sock,$blocking) = @_;
+				my ($call,$sock,$rb) = @_;
 				DEBUG( 100,"rtp_cleanup: remove socket %d",fileno($sock));
 				$call->{loop}->delFD( $sock );
-				$sock->blocking(1) if $blocking;
-			}, $call,$sock,$was_blocking ];
+				$sock->blocking(1) if $rb;
+			}, $call,$sock,$reset_to_blocking ];
 		}
 
 		# on RTP inactivity for at least 10 seconds close connection
@@ -136,10 +140,11 @@ sub media_send_recv {
 				while (1) {
 					my $buf = _receive_rtp( $sock,$writeto,$targs,$didit ); 
 					defined($buf) or return;
+					CAN_NONBLOCKING or return;
 				}
 			};
 			$call->{loop}->addFD( $sock, [ $receive,$writeto,{},\$didit ] );
-			my $was_blocking = $sock->blocking(0);
+			my $reset_to_blocking = CAN_NONBLOCKING && $sock->blocking(0);
 
 			# sending need to be done with a timer
 			# ! $addr == call on hold
@@ -149,18 +154,19 @@ sub media_send_recv {
 					0, # start immediatly
 					[ \&_send_rtp,$s_sock,$call->{loop},$addr,$readfrom, {
 						repeat => $repeat || 1,
-						cb_done => [ sub { invoke_callback(@_) }, $cb_done, $call ]
+						cb_done => [ sub { invoke_callback(@_) }, $cb_done, $call ],
+						rtp_param => $args->{rtp_param},
 					}],
-					160/8000, # 8000 bytes per second, 160 bytes per sample
+					$args->{rtp_param}[2], # repeat timer
 					'rtpsend',
 				);
 
 				push @{ $call->{ rtp_cleanup }}, [ sub {
-					my ($call,$sock,$timer,$wb) = @_;
+					my ($call,$sock,$timer,$rb) = @_;
 					$call->{loop}->delFD( $sock );
-					$sock->blocking(1) if $wb;
+					$sock->blocking(1) if $rb;
 					$timer->cancel();
-				}, $call,$sock,$timer,$was_blocking ];
+				}, $call,$sock,$timer,$reset_to_blocking ];
 			}
 		}
 
@@ -259,7 +265,7 @@ sub _receive_rtp {
 ###########################################################################
 # Helper to read  RTP data from file (PCMU 8000) and send them through
 # the RTP socket
-# Args: ($sock,$loop,$addr,$readfrom,$targs)
+# Args: ($sock,$loop,$addr,$readfrom,$targs,$timer)
 #   $sock: RTP socket
 #   $loop: event loop (used for looptime for timestamp)
 #   $addr: where to send data
@@ -269,6 +275,7 @@ sub _receive_rtp {
 #     send (<=0 means forever) and 'cb_done' holds a [\&sub,@arg] callback
 #     to end the call after sending all data
 #     'repeat' makes only sense if $readfrom is filename
+#   $timer: timer which gets canceled once all data are send
 # Return: NONE
 ###########################################################################
 sub _send_rtp {
@@ -302,7 +309,8 @@ sub _send_rtp {
 				open( $fd,'<',$readfrom ) || die $!;
 				$targs->{fd} = $fd;
 			}
-			last if read( $fd,$buf,160 ) == 160;
+			my $size = $targs->{rtp_param}[1]; # 160 for PCMU/8000
+			last if read( $fd,$buf,$size ) == $size;
 			# try to reopen file
 			close($fd);
 			$targs->{fd} = undef;
@@ -312,10 +320,11 @@ sub _send_rtp {
 	$buf || die $!;
 
 	# add RTP header
-	my $now = $loop->looptime;
-	my $timestamp = ( $now * 10_000 ) % 2**32; # 10*ms precision in 32 bit
 	$targs->{wseq}++;
 	my $seq = $targs->{wseq};
+	# 32 bit timestamp based on seq and packet size
+	# FIXME: it assumes here that packet size == number of samples which is true only for 8bit
+	my $timestamp = ( $targs->{rtp_param}[1] * $seq ) % 2**32; 
 
 	if (0) {
 		my ($fp,$fa) = unpack_sockaddr_in( getsockname($sock) );
@@ -327,11 +336,12 @@ sub _send_rtp {
 
 	my $header = pack('CCnNN',
 		0b10000000, # Version 2
-		0b00000000, # PMCU 8000
+		$targs->{rtp_param}[0], # 0 == PMCU 8000
 		$seq, # sequence
 		$timestamp,
 		0x1234,    # source ID
 	);
+	DEBUG( 100,"send %d bytes to RTP", length($buf));
 	send( $sock,$header.$buf,0,$addr ) || die $!;
 }
 
